/* hexagon chess -- fr 2003 
 * GNU Prolog 1.2.16 (nasm 0.98.38)
 */

/* erfordert Voberhandlung mit C-Preprocessor:
   #define BRETT_3F  oder #define BRETT_3W ! */


/* VERSION 0.2

  Einschraenkungen/Probleme:
  
  Bauern: kein Doppelzug, kein en-passant 
  Bauern: Umwandlung immer in Dame
  keine Rochade

  Sind die einfach uebertragenen Schachregeln so ueberhaupt sinnvoll ?
  - Die Dame kann z.B. sofort losgehen - (bei a32)

  Kein Patt bei Zugfolgenwiederholung.

  Der Algorithmus des Computerspielers ist noch zu simpel:
  Suchtiefe nur 2 Halbzuege.
  Keine Zugsortierung.
  Keine Iterative Suche.
  Stellungsbewertung zu simpel.
  ...
  
*/


/* Globales Ziel: */
:- initialization(spiel).

/* Hilfspraedikate: */
not(C) :- call(C), !, fail.
not(_).

/* Optionen und Defaults: */
:- dynamic(ansi/1).
:- dynamic(set/1).
:- dynamic(beginner/1).
:- dynamic(mensch/1).
:- dynamic(tiefe/2).
:- dynamic(dbg/1).
:- dynamic(anf/1).
ansi(yes).
set(no).
beginner(we).
mensch(we).
tiefe(2,4).
dbg(no).
anf(anormal).


/* Farben : we = weiss,   sw = schwarz, ro = rot
/* Figuren: k  = Koenig,  d  = Dame,    s  = Springer,
            l  = Laeufer, t  = Turm,    b  = Bauer     */

#ifdef BRETT_3F
/* Das Brett wird durch Aufzaehlung seiner Felder definiert: */
                     feld(sw,231). feld(sw,232). feld(sw,233). feld(sw,234).
                            feld(ro,220). feld(ro,221). feld(ro,222). feld(ro,223).
                     feld(we,208). feld(we,209). feld(we,210). feld(we,211).
              feld(sw,196). feld(sw,197). feld(sw,198). feld(sw,199). feld(sw,200).
                     feld(ro,185). feld(ro,186). feld(ro,187). feld(ro,188). feld(ro,189).
              feld(we,173). feld(we,174). feld(we,175). feld(we,176). feld(we,177).
       feld(sw,161). feld(sw,162). feld(sw,163). feld(sw,164). feld(sw,165). feld(sw,166).
              feld(ro,150). feld(ro,151). feld(ro,152). feld(ro,153). feld(ro,154). feld(ro,155).
       feld(we,138). feld(we,139). feld(we,140). feld(we,141). feld(we,142). feld(we,143).
feld(sw,126). feld(sw,127). feld(sw,128). feld(sw,129). feld(sw,130). feld(sw,131). feld(sw,132).
       feld(ro,115). feld(ro,116). feld(ro,117). feld(ro,118). feld(ro,119). feld(ro,120). feld(ro,121).
feld(we,103). feld(we,104). feld(we,105). feld(we,106). feld(we,107). feld(we,108). feld(we,109).
       feld(sw, 92). feld(sw, 93). feld(sw, 94). feld(sw, 95). feld(sw, 96). feld(sw, 97).
              feld(ro, 81). feld(ro, 82). feld(ro, 83). feld(ro, 84). feld(ro, 85). feld(ro, 86).
       feld(we, 69). feld(we, 70). feld(we, 71). feld(we, 72). feld(we, 73). feld(we, 74).
              feld(sw, 58). feld(sw, 59). feld(sw, 60). feld(sw, 61). feld(sw, 62).
                     feld(ro, 47). feld(ro, 48). feld(ro, 49). feld(ro, 50). feld(ro, 51).
              feld(we, 35). feld(we, 36). feld(we, 37). feld(we, 38). feld(we, 39).
                     feld(sw, 24). feld(sw, 25). feld(sw, 26). feld(sw, 27).
                            feld(ro, 13). feld(ro, 14). feld(ro, 15). feld(ro, 16).
                     feld(we,  1). feld(we,  2). feld(we,  3). feld(we,  4).

/* Ausgabe des Brettes: */
ausgabe(S) :-
write('            '),     aus(S,231), aus(S,232), aus(S,233), aus(S,234), nl,
write('                '), aus(S,220), aus(S,221), aus(S,222), aus(S,223), nl,
write('            '),     aus(S,208), aus(S,209), aus(S,210), aus(S,211), nl,
write('        '),         aus(S,196), aus(S,197), aus(S,198), aus(S,199), aus(S,200), nl,
write('            '),     aus(S,185), aus(S,186), aus(S,187), aus(S,188), aus(S,189), nl,
write('        '),         aus(S,173), aus(S,174), aus(S,175), aus(S,176), aus(S,177), nl,
write('    '),             aus(S,161), aus(S,162), aus(S,163), aus(S,164), aus(S,165), aus(S,166), nl,
write('        '),         aus(S,150), aus(S,151), aus(S,152), aus(S,153), aus(S,154), aus(S,155), nl,
write('    '),             aus(S,138), aus(S,139), aus(S,140), aus(S,141), aus(S,142), aus(S,143), nl,
                           aus(S,126), aus(S,127), aus(S,128), aus(S,129), aus(S,130), aus(S,131), aus(S,132), nl,
write('    '),             aus(S,115), aus(S,116), aus(S,117), aus(S,118), aus(S,119), aus(S,120), aus(S,121), nl,
                           aus(S,103), aus(S,104), aus(S,105), aus(S,106), aus(S,107), aus(S,108), aus(S,109), nl,
write('    '),             aus(S, 92), aus(S, 93), aus(S, 94), aus(S, 95), aus(S, 96), aus(S, 97), nl,
write('        '),         aus(S, 81), aus(S, 82), aus(S, 83), aus(S, 84), aus(S, 85), aus(S, 86), nl,
write('    '),             aus(S, 69), aus(S, 70), aus(S, 71), aus(S, 72), aus(S, 73), aus(S, 74), nl,
write('        '),         aus(S, 58), aus(S, 59), aus(S, 60), aus(S, 61), aus(S, 62), nl,
write('            '),     aus(S, 47), aus(S, 48), aus(S, 49), aus(S, 50), aus(S, 51), nl,
write('        '),         aus(S, 35), aus(S, 36), aus(S, 37), aus(S, 38), aus(S, 39), nl,
write('            '),     aus(S, 24), aus(S, 25), aus(S, 26), aus(S, 27), nl,
write('                '), aus(S, 13), aus(S, 14), aus(S, 15), aus(S, 16), nl,
write('            '),     aus(S,  1), aus(S,  2), aus(S,  3), aus(S,  4), nl, nl.

aus(S,FELD) :- figur(S,FARBE,FIGUR,FELD), !, feldfarbe(FELD), format('  %3d:',[FELD]), farbe(FARBE), wspieler(FARBE), wfigur(FIGUR).
aus(_,FELD) :- feldfarbe(FELD), format('  %3d   ',[FELD]).

/* Anfangsstellungen: */
anfang(anormal,
[f(we,t,  1),f(we,s, 13),f(we,l,  2),f(we,d, 14),f(we,k,  3),f(we,l, 15),f(we,s,  4),f(we,t, 16),
 f(we,b, 24),f(we,b, 36),f(we,b, 25),f(we,b, 37),f(we,b, 26),f(we,b, 38),f(we,b, 27),f(we,b, 39),
 f(sw,t,231),f(sw,s,220),f(sw,l,232),f(sw,d,221),f(sw,k,233),f(sw,l,222),f(sw,s,234),f(sw,t,223),
 f(sw,b,208),f(sw,b,197),f(sw,b,209),f(sw,b,198),f(sw,b,210),f(sw,b,199),f(sw,b,211),f(sw,b,200)]).

anfang(a0,[]).

anfang(atest,[f(we,k,  3),f(we,t,  1),f(we,d, 15),f(we,t, 26),
              f(sw,k,233),f(sw,b,198),f(sw,b,210),f(sw,b,199)]).

/* Die Zuege der Spielfiguren:
   l1 = Laeufer-Einzelschritt, t1 = Turm-Einzelschritt 
   lm = Laeufer-Mehrschritt,   tm = Turm-Mehrschritt  */
l1(VON,NACH,  1) :- NACH is VON +  1, feld(_,NACH).
l1(VON,NACH, -1) :- NACH is VON -  1, feld(_,NACH).
l1(VON,NACH, 34) :- NACH is VON + 34, feld(_,NACH).
l1(VON,NACH,-34) :- NACH is VON - 34, feld(_,NACH).
l1(VON,NACH, 35) :- NACH is VON + 35, feld(_,NACH).
l1(VON,NACH,-35) :- NACH is VON - 35, feld(_,NACH).

t1(VON,NACH, 11) :- NACH is VON + 11, feld(_,NACH).
t1(VON,NACH,-11) :- NACH is VON - 11, feld(_,NACH).
t1(VON,NACH, 12) :- NACH is VON + 12, feld(_,NACH).
t1(VON,NACH,-12) :- NACH is VON - 12, feld(_,NACH).
t1(VON,NACH, 23) :- NACH is VON + 23, feld(_,NACH).
t1(VON,NACH,-23) :- NACH is VON - 23, feld(_,NACH).

springer(VON,NACH) :- NACH is VON + 10, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 10, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 13, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 13, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 45, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 45, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 47, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 47, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 57, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 57, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 58, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 58, feld(_,NACH).

bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 23, feld(_,NACH), leer( STELLUNG,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 34, feld(_,NACH), figur(STELLUNG,sw,_,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 35, feld(_,NACH), figur(STELLUNG,sw,_,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 23, feld(_,NACH), leer( STELLUNG,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 34, feld(_,NACH), figur(STELLUNG,we,_,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 35, feld(_,NACH), figur(STELLUNG,we,_,NACH).

bauernumwandlung(we,b,FELD,d) :- FELD > 219, !.
bauernumwandlung(sw,b,FELD,d) :- FELD <  17, !.
bauernumwandlung(_,FIGUR,_,FIGUR).
#endif
#ifdef BRETT_3W
/* Das Brett wird durch Aufzaehlung seiner Felder definiert: */
                                                 feld(ro,163). feld(sw,164). feld(we,165). feld(ro,166).
                                          feld(sw,149). feld(we,150). feld(ro,151). feld(sw,152). feld(we,153).
                     feld(sw,134). feld(we,135). feld(ro,136). feld(sw,137). feld(we,138). feld(ro,139). feld(sw,140). feld(we,141).
feld(sw,119). feld(we,120). feld(ro,121). feld(sw,122). feld(we,123). feld(ro,124). feld(sw,125). feld(we,126). feld(ro,127). feld(sw,128). feld(we,129).
       feld(ro,106). feld(sw,107). feld(we,108). feld(ro,109). feld(sw,110). feld(we,111). feld(ro,112). feld(sw,113). feld(we,114). feld(ro,115).
feld(sw, 92). feld(we, 93). feld(ro, 94). feld(sw, 95). feld(we, 96). feld(ro, 97). feld(sw, 98). feld(we, 99). feld(ro,100). feld(sw,101). feld(we,102).
       feld(ro, 79). feld(sw, 80). feld(we, 81). feld(ro, 82). feld(sw, 83). feld(we, 84). feld(ro, 85). feld(sw, 86). feld(we, 87). feld(ro, 88).
feld(sw, 65). feld(we, 66). feld(ro, 67). feld(sw, 68). feld(we, 69). feld(ro, 70). feld(sw, 71). feld(we, 72). feld(ro, 73). feld(sw, 74). feld(we, 75).
       feld(ro, 52). feld(sw, 53). feld(we, 54). feld(ro, 55). feld(sw, 56). feld(we, 57). feld(ro, 58). feld(sw, 59). feld(we, 60). feld(ro, 61).
feld(sw, 38). feld(we, 39). feld(ro, 40). feld(sw, 41). feld(we, 42). feld(ro, 43). feld(sw, 44). feld(we, 45). feld(ro, 46). feld(sw, 47). feld(we, 48).
                     feld(sw, 26). feld(we, 27). feld(ro, 28). feld(sw, 29). feld(we, 30). feld(ro, 31). feld(sw, 32). feld(we, 33).
                                          feld(sw, 14). feld(we, 15). feld(ro, 16). feld(sw, 17). feld(we, 18).
                                                 feld(ro,  1). feld(sw,  2). feld(we,  3). feld(ro,  4).

/* Ausgabe des Brettes: */
ausgabe(S) :-
write('                     '), aus(S,163), aus(S,164), aus(S,165), aus(S,166), nl, nl,
write('                  '),    aus(S,149), aus(S,150), aus(S,151), aus(S,152), aus(S,153), nl, nl,
write('         '),             aus(S,134), aus(S,135), aus(S,136), aus(S,137), aus(S,138), aus(S,139), aus(S,140), aus(S,141), nl, nl,
                                aus(S,119), aus(S,120), aus(S,121), aus(S,122), aus(S,123), aus(S,124), aus(S,125), aus(S,126), aus(S,127), aus(S,128), aus(S,129), nl, nl,
write('   '),                   aus(S,106), aus(S,107), aus(S,108), aus(S,109), aus(S,110), aus(S,111), aus(S,112), aus(S,113), aus(S,114), aus(S,115), nl, nl,
                                aus(S, 92), aus(S, 93), aus(S, 94), aus(S, 95), aus(S, 96), aus(S, 97), aus(S, 98), aus(S, 99), aus(S,100), aus(S,101), aus(S,102), nl, nl,
write('   '),                   aus(S, 79), aus(S, 80), aus(S, 81), aus(S, 82), aus(S, 83), aus(S, 84), aus(S, 85), aus(S, 86), aus(S, 87), aus(S, 88), nl, nl,
                                aus(S, 65), aus(S, 66), aus(S, 67), aus(S, 68), aus(S, 69), aus(S, 70), aus(S, 71), aus(S, 72), aus(S, 73), aus(S, 74), aus(S, 75), nl, nl,
write('   '),                   aus(S, 52), aus(S, 53), aus(S, 54), aus(S, 55), aus(S, 56), aus(S, 57), aus(S, 58), aus(S, 59), aus(S, 60), aus(S, 61), nl, nl,
                                aus(S, 38), aus(S, 39), aus(S, 40), aus(S, 41), aus(S, 42), aus(S, 43), aus(S, 44), aus(S, 45), aus(S, 46), aus(S, 47), aus(S, 48), nl, nl,
write('         '),             aus(S, 26), aus(S, 27), aus(S, 28), aus(S, 29), aus(S, 30), aus(S, 31), aus(S, 32), aus(S, 33), nl, nl,
write('                  '),    aus(S, 14), aus(S, 15), aus(S, 16), aus(S, 17), aus(S, 18), nl, nl,
write('                     '), aus(S,  1), aus(S,  2), aus(S,  3), aus(S,  4), nl, nl, nl.

aus(S,FELD) :- figur(S,FARBE,FIGUR,FELD), !, feldfarbe(FELD), format(' %3d',[FELD]), farbe(FARBE), wspieler(FARBE), wfigur(FIGUR).
aus(_,FELD) :- feldfarbe(FELD), format(' %3d  ',[FELD]).

/* Anfangsstellungen: */
anfang(anormal,
[f(we,t,  1),f(we,d,  2),f(we,k,  3),f(we,t,  4),
 f(we,s, 14),f(we,l, 15),f(we,l, 16),f(we,l, 17),f(we,s, 18),
 f(we,b, 26),f(we,b, 27),f(we,b, 28),f(we,b, 29),f(we,b, 30),f(we,b, 31),f(we,b, 32),f(we,b, 33),
 f(we,b, 42),f(we,b, 43),f(we,b, 44),
 f(sw,t,163),f(sw,k,164),f(sw,d,165),f(sw,t,166),
 f(sw,s,149),f(sw,l,150),f(sw,l,151),f(sw,l,152),f(sw,s,153),
 f(sw,b,134),f(sw,b,135),f(sw,b,136),f(sw,b,137),f(sw,b,138),f(sw,b,139),f(sw,b,140),f(sw,b,141),
 f(sw,b,123),f(sw,b,124),f(sw,b,125)]).

anfang(a0,[]).

anfang(atest,[f(we,k,  3),f(we,t,  1),f(we,d, 2),f(we,t, 4),
              f(sw,k,164),f(sw,b,136),f(sw,b,137),f(sw,b,138)]).

/* Die Zuege der Spielfiguren:
   l1 = Laeufer-Einzelschritt, t1 = Turm-Einzelschritt 
   lm = Laeufer-Mehrschritt,   tm = Turm-Mehrschritt  */
l1(VON,NACH, 12) :- NACH is VON + 12, feld(_,NACH).
l1(VON,NACH,-12) :- NACH is VON - 12, feld(_,NACH).
l1(VON,NACH, 15) :- NACH is VON + 15, feld(_,NACH).
l1(VON,NACH,-15) :- NACH is VON - 15, feld(_,NACH).
l1(VON,NACH, 27) :- NACH is VON + 27, feld(_,NACH).
l1(VON,NACH,-27) :- NACH is VON - 27, feld(_,NACH).

t1(VON,NACH,  1) :- NACH is VON +  1, feld(_,NACH).
t1(VON,NACH, -1) :- NACH is VON -  1, feld(_,NACH).
t1(VON,NACH, 13) :- NACH is VON + 13, feld(_,NACH).
t1(VON,NACH,-13) :- NACH is VON - 13, feld(_,NACH).
t1(VON,NACH, 14) :- NACH is VON + 14, feld(_,NACH).
t1(VON,NACH,-14) :- NACH is VON - 14, feld(_,NACH).

springer(VON,NACH) :- NACH is VON + 11, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 11, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 16, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 16, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 25, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 25, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 29, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 29, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 40, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 40, feld(_,NACH).
springer(VON,NACH) :- NACH is VON + 41, feld(_,NACH).
springer(VON,NACH) :- NACH is VON - 41, feld(_,NACH).

bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 13, feld(_,NACH), leer( STELLUNG,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 14, feld(_,NACH), leer( STELLUNG,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 26, feld(_,NACH), figur(STELLUNG,sw,_,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 27, feld(_,NACH), figur(STELLUNG,sw,_,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,we,b,VON), NACH is VON + 28, feld(_,NACH), figur(STELLUNG,sw,_,NACH).

bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 13, feld(_,NACH), leer( STELLUNG,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 14, feld(_,NACH), leer( STELLUNG,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 26, feld(_,NACH), figur(STELLUNG,we,_,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 27, feld(_,NACH), figur(STELLUNG,we,_,NACH).
bauer(VON,NACH,STELLUNG) :- figur(STELLUNG,sw,b,VON), NACH is VON - 28, feld(_,NACH), figur(STELLUNG,we,_,NACH).

bauernumwandlung(we,b,FELD,d) :- FELD > 162, !.
bauernumwandlung(sw,b,FELD,d) :- FELD <   5, !.
bauernumwandlung(_,FIGUR,_,FIGUR).
#endif

lm(VON,NACH,RICHTUNG,_)        :- l1(VON,NACH,RICHTUNG).
lm(VON,NACH,RICHTUNG,STELLUNG) :- l1(VON,P,RICHTUNG), leer(STELLUNG,P), lm(P,NACH,RICHTUNG,STELLUNG).

tm(VON,NACH,RICHTUNG,_)        :- t1(VON,NACH,RICHTUNG).
tm(VON,NACH,RICHTUNG,STELLUNG) :- t1(VON,P,RICHTUNG), leer(STELLUNG,P), tm(P,NACH,RICHTUNG,STELLUNG).


laeufer(VON,NACH,STELLUNG)  :- lm(VON,NACH,_,STELLUNG).

turm(VON,NACH,STELLUNG)     :- tm(VON,NACH,_,STELLUNG).

dame(VON,NACH,STELLUNG)     :- lm(VON,NACH,_,STELLUNG).
dame(VON,NACH,STELLUNG)     :- tm(VON,NACH,_,STELLUNG).

koenig(VON,NACH)   :- l1(VON,NACH,_).
koenig(VON,NACH)   :- t1(VON,NACH,_).


/* mit der Figur als Variable: */
kannziehen(STELLUNG,b,VON,NACH) :- bauer(VON,NACH,STELLUNG).
kannziehen(STELLUNG,t,VON,NACH) :- turm(VON,NACH,STELLUNG).
kannziehen(STELLUNG,l,VON,NACH) :- laeufer(VON,NACH,STELLUNG).
kannziehen(STELLUNG,d,VON,NACH) :- dame(VON,NACH,STELLUNG).
kannziehen(_,s,VON,NACH)        :- springer(VON,NACH).
kannziehen(_,k,VON,NACH)        :- koenig(VON,NACH).


/* Stellungen sind Listen von Tripeln: f(Farbe,Figur,Feld) */

/* Wo steht was fuer eine Figur: */
figur(STELLUNG,FARBE,FIGUR,FELD) :- member(f(FARBE,FIGUR,FELD),STELLUNG).

leer(STELLUNG,FELD) :- not(figur(STELLUNG,_,_,FELD)).


/* Zug ausfuehren: */
zug(STELLUNG,VON,NACH,[f(FARBE,FIGUR2,NACH)|S2],[GFIG]) :-
            figur(STELLUNG,FARBE,FIGUR,VON),
            gegner(FARBE,GEGNER),
            figur(STELLUNG,GEGNER,GFIG,NACH), !,
            delete(STELLUNG,f(FARBE,FIGUR,VON),S1),
            delete(S1,f(GEGNER,GFIG,NACH),S2),
            bauernumwandlung(FARBE,FIGUR,NACH,FIGUR2).
zug(STELLUNG,VON,NACH,[f(FARBE,FIGUR2,NACH)|S1],[]) :-
            figur(STELLUNG,FARBE,FIGUR,VON),
            delete(STELLUNG,f(FARBE,FIGUR,VON),S1),
            bauernumwandlung(FARBE,FIGUR,NACH,FIGUR2).


/* moegliche Zuege: */
potentiellerzug(STELLUNG,SPIELER,z(FIGUR,VON,NACH)) :-
            member(f(SPIELER,FIGUR,VON),STELLUNG),
            kannziehen(STELLUNG,FIGUR,VON,NACH),
            not(figur(STELLUNG,SPIELER,_,NACH)).

/* nicht im Schach stehenbleiben: */
erlaubterzug(STELLUNG,SPIELER,z(FIGUR,VON,NACH)) :-
            potentiellerzug(STELLUNG,SPIELER,z(FIGUR,VON,NACH)),
            zug(STELLUNG,VON,NACH,STELLUNGNEU,_),
            not(imschach(STELLUNGNEU,SPIELER)).

/* ***
schlagzug(STELLUNG,SPIELER,z(FIGUR,VON,NACH)) :-
            potentiellerzug(STELLUNG,SPIELER,z(FIGUR,VON,NACH)),
            zug(STELLUNG,VON,NACH,STELLUNGNEU,SCHLAGLISTE),
            SCHLAGLISTE \== [],
            not(imschach(STELLUNGNEU,SPIELER)).
*** */

/* Schach / Matt ? */
imschach(STELLUNG,SPIELER) :-
            gegner(SPIELER,GEGNER),
            potentiellerzug(STELLUNG,GEGNER,z(_,_,NACH)),
            figur(STELLUNG,SPIELER,k,NACH).


matt(STELLUNG,SPIELER) :-
            findall(Z,erlaubterzug(STELLUNG,SPIELER,Z),[]),
            imschach(STELLUNG,SPIELER).

/* ***
patt(STELLUNG,SPIELER) :-
            findall(Z,erlaubterzug(STELLUNG,SPIELER,Z),[]),
            not(imschach(STELLUNG,SPIELER)).
*** */


/* Die Gegner: */
gegner(we,sw).
gegner(sw,we).


/* Der Computerspieler: */
computerzug(STELLUNG,SPIELER,VON,NACH,WERT) :-
            minimax(STELLUNG,SPIELER,z(_,VON,NACH),WERT,-20000,+20000,0,[]).


/* Minimax-Suche mit Alpha-Beta-Pruning.
 
 Der Wert einer Stellung gibt an, wer (wohl) gewinnt. 
 Wert > 0 heisst: Die Stellung ist besser fuer Weiss, 
 Wert < 0 : besser fuer Schwarz. Wert = 0 : ausgeglichen. 
 
 Bei Gewin einer Seite oder bei Erreichen der maximalen Suchtiefe wird 
 der Wert berechnet. Das ergibt viele virtuelle Stellungen mit ihren Werten, 
 die aber noch ein paar Zuege entfernt sind. Wie kommt man zum besten Wert ? 
 
 Liegen z.B. die Werte fuer alle Stellungen direkt nach dem anstehenden 
 (Halb-)Zug vor und ist etwa Weiss am Zug, dann wird Weiss natuerlich den Zug 
 waehlen, der zum groessten Wert fuehrt. Also kann man sagen, der Wert dieser 
 Stellung (vor dem Zug) ist gleich dem Maxmimum der Werte nach dem Zug - 
 denn warum sollte Weiss einen schlechteren Zug machen. 
 
 Vor Weiss ist Schwarz an der Reihe. Schwarz hat die Auswahl aus mehreren 
 Zuegen, von den jeder zu einer anderen Variante fuer Weiss fuehrt. 
 In jeder dieser Varianten hat Weiss seinen besten Zug durch Maxmimumbildung 
 bestimmt. Schwarz wird nun den Zug machen, der zu der fuer Weiss 
 schlechtesten - und damit fuer ihen besten - Variante fuehrt: den Zug, 
 der dem Minimum der Werte der besten weissen Zuege entspricht, usw. 
 
 Der Wert einer Stellung, in der Weiss am Zuge ist, ist gleich dem Maximum 
 der Werte der Stellungen nach allen moeglichen Zuegen von Weiss. 
 
 Der Wert einer Stellung, in der Schwarz am Zuge ist, ist gleich dem Minimum 
 der Werte der Stellungen nach allen moeglichen Zuegen von Schwarz. 
 
 Und es wird jeweils der Zug ausgefuehrt, der dem Maximum bzw. Minimum 
 entspricht. Dies ist die Minimax-Suche.
 
 Nun zum Alpha-Beta-Pruning. 
 
 Das Prinzip bleibt so, wie beschrieben. Waehrend der Suche, bei der ja 
 Weiss das Maximum der Werte seiner Zuege bilden muss, merkt Weiss sich 
 den bis dahin besten Wert (= Alpha).
 
 In jeder der Varianten werden mehrere Folgezuege von Schwarz untersucht.
 Wenn nun bei einem dieser Folgezuege ein Wert < Alpha ausftritt, dann wird
 der Wert dieser Variante auch < Alpha sein. Denn Schwarz wird diesen
 Zug mit dem Wert < Alpha ausfuehren - oder einen fuer ihn noch besseren,
 also mit noch kleinerem Wert. 
 
 Das bedeutet, die Untersuchung dieser Variante kann abgebrochen werden, 
 sie kann nicht das Maximum werden.
 
 Das Gleiche gilt umgekehrt fuer Schwarz: Schwarz merkt sich den besten =
 kleinsten Wert (Beta). Falls in einer Variante Weiss einen Wert > Beta
 erreichen kann, kann die Berechnung der Variante abgebrochen werden.
*/


/* minimax(STELLUNG,SPIELER,BESTZUG,WERT,ALPHA,BETA,TIEFE,SCHLAGLISTE) */
minimax(STELLUNG,SPIELER,_,WERT,_,_,TIEFE,[]) :-  
            TIEFE > 1, !,                     /* Abbruch bei Ruhezug */
            wert(STELLUNG,BALANCE),
            zuglistenwert(STELLUNG,SPIELER,ZLW),
            WERT is BALANCE + ZLW,
            dbgwert(wert,WERT,TIEFE).
minimax(STELLUNG,SPIELER,_,WERT,_,_,TIEFE,_) :-   
            TIEFE > 3, !,                     /* Abbruch auf jeden Fall */
            wert(STELLUNG,BALANCE),
            zuglistenwert(STELLUNG,SPIELER,ZLW),
            WERT is BALANCE + ZLW,
            dbgwert(wert,WERT,TIEFE).
/* ***
minimax(STELLUNG,SPIELER,BESTZUG,WERT,ALPHA,BETA,TIEFE,_) :-
            TIEFE > 1, !                     
            findall(Z,schlagzug(STELLUNG,SPIELER,Z),ZUGLISTE), !,
            farbe(SPIELER), TIEFENCHAR is TIEFE + 32, put_code(TIEFENCHAR),
            dbgliste(SPIELER,ZUGLISTE,TIEFE),
            besterzug(STELLUNG,SPIELER,ZUGLISTE,BESTZUG,WERT,ALPHA,BETA,TIEFE),
            dbgbester(SPIELER,BESTZUG,WERT,TIEFE).
*** */

minimax(STELLUNG,SPIELER,BESTZUG,WERT,ALPHA,BETA,TIEFE,_) :-
            findall(Z,erlaubterzug(STELLUNG,SPIELER,Z),ZUGLISTE), !,
            minimaxliste(STELLUNG,SPIELER,ZUGLISTE,BESTZUG,WERT,ALPHA,BETA,TIEFE).

/* erstmal die Faelle mit leerer Liste erlaubter Zuege:
   im Schach -> Matt (Wert=+-30000), sonst Patt (Wert 0).
   Tiefe 0: Ende des Spieles. */
minimaxliste(STELLUNG,SPIELER,ZUGLISTE,_,WERT,_,_,TIEFE) :-
            ZUGLISTE == [],
            imschach(STELLUNG,SPIELER), !,                 /* matt */
            mattwert(SPIELER,WERT),
            dbgwert('w matt  wert',WERT,TIEFE),
            spielende(TIEFE,SPIELER,'Matt').
minimaxliste(STELLUNG,SPIELER,ZUGLISTE,_,0,_,_,TIEFE) :- 
            ZUGLISTE == [],
            not(imschach(STELLUNG,SPIELER)), !,            /* patt */
            dbgwert('patt  wert',0,TIEFE),
            spielende(TIEFE,SPIELER,'Patt').
/* nun der Normalfall - mindestens noch ein erlaubter Zug: */
minimaxliste(STELLUNG,SPIELER,ZUGLISTE,BESTZUG,WERT,ALPHA,BETA,TIEFE) :-
            farbe(SPIELER), TIEFENCHAR is TIEFE + 32, put_code(TIEFENCHAR),
            dbgliste(SPIELER,ZUGLISTE,TIEFE),
            besterzug(STELLUNG,SPIELER,ZUGLISTE,BESTZUG,WERT,ALPHA,BETA,TIEFE),
            dbgbester(SPIELER,BESTZUG,WERT,TIEFE).


/* besterzug(STELLUNG,SPIELER,ZUGLISTE,BESTZUG,WERT,ALPHA,BETA,TIEFE) */
besterzug(STELLUNG,SPIELER,[z(FIGUR,VON,NACH)|T],BESTZUG,WERT,ALPHA,BETA,TIEFE) :-
            zug(STELLUNG,VON,NACH,STELLUNGNEU,SCHLAGLISTE), !,
            dbgzug(SPIELER,FIGUR,VON,NACH,TIEFE),
            gegner(SPIELER,GEGNER),
            TIEFENEU is TIEFE + 1,
            minimax(STELLUNGNEU,GEGNER,_,W1,ALPHA,BETA,TIEFENEU,SCHLAGLISTE), !,
            bessererzug(STELLUNG,SPIELER,T,z(FIGUR,VON,NACH),W1,BESTZUG,WERT,ALPHA,BETA,TIEFE).


/* bessererzug(STELLUNG,SPIELER,ZUGLISTE,BISHERBESTERZUG,BISHERBESTERWERT,BESTZUG,WERT,ALPHA,BETA,TIEFE) */

/* nur ein moeglicher Zug: */
bessererzug(_,_,[],BESTZUG,BESTWERT,BESTZUG,BESTWERT,_,_,_).

/* Weiss: Wert eines Folgezuges (Schwarz !) < Alpha -> Abschneiden,
   einfach diesen (schlechten) Wert hochreichen; ... und umgekehrt */
bessererzug(_,sw,_,ZUG,WERT,ZUG,WERT,ALPHA,_,TIEFE) :-
            WERT < ALPHA, !,
            dbgalphaschnitt(ZUG,WERT,ALPHA,TIEFE).
bessererzug(_,we,_,ZUG,WERT,ZUG,WERT,_,BETA,TIEFE)  :-
            WERT > BETA,  !,
            dbgbetaschnitt(ZUG,WERT,BETA,TIEFE).

/* sonst (leider) nicht abschneiden, nur Alpha und Beta korrigieren: */
bessererzug(STELLUNG,SPIELER,ZUGLISTE,Z1,W1,BESTZUG,WERT,ALPHA,BETA,TIEFE) :-
            neuegrenzen(SPIELER,W1,ALPHA,BETA,A1,B1,TIEFE),
            besterzug(STELLUNG,SPIELER,ZUGLISTE,Z2,W2,A1,B1,TIEFE),
            minormax(SPIELER,W1,Z1,W2,Z2,WERT,BESTZUG).


/* Weiss: Alpha = groesster gefundener Wert, Schwarz: Beta = kleinster ...*/
neuegrenzen(we,WERT,ALPHA,BETA,WERT ,BETA,TIEFE) :-
            WERT > ALPHA, !,
            dbgwert(alpha,WERT,TIEFE).
neuegrenzen(sw,WERT,ALPHA,BETA,ALPHA,WERT,TIEFE) :-
            WERT < BETA , !,
            dbgwert(beta,WERT,TIEFE).
neuegrenzen(_,_,ALPHA,BETA,ALPHA,BETA,_) /* :- format(' alpha=%d beta=%d~n',[ALPHA,BETA]) */.


/* max fuer weiss, min fuer schwarz : */
minormax(we,W1,Z1,W2,_,W1,Z1) :- W1 >= W2, !.
minormax(sw,W1,Z1,W2,_,W1,Z1) :- W1 =< W2, !.
minormax(_,_,_,W2,Z2,W2,Z2).


/* Stellungsbewertung: Weiss positiv, Schwarz negativ.
   Hier einfach nur: Materialbalance
   Beim Aufruf von wert() wird noch die Anzahl der moeglichen
   Zuege addiert (-> zuglistenwert)
*/
wert([],0).
wert([f(FARBE,FIGUR,_)|T],WERT) :-
            wertfigur(FARBE,FIGUR,WH),
            wert(T,WT),
            WERT is WH + WT.

wertfigur(sw,FIGUR,WS) :- wertfigur(we,FIGUR,WW), WS is -WW.
wertfigur(we,b,100).
wertfigur(we,s,325).
wertfigur(we,l,325).
wertfigur(we,t,500).
wertfigur(we,d,900).
wertfigur(we,k,20000).

mattwert(we,-30000).
mattwert(sw,+30000).

zuglistenwert(STELLUNG,we,WERT) :-
            findall(Z,potentiellerzug(STELLUNG,we,Z),ZUGLISTE),
            length(ZUGLISTE,WERT).
zuglistenwert(STELLUNG,sw,WERT) :-
            findall(Z,potentiellerzug(STELLUNG,sw,Z),ZUGLISTE),
            length(ZUGLISTE,LEN),
            WERT is -LEN.


/* Eingabe: */
zugeingabe(SPIELER,VON,NACH,_,_) :-
            set(no),
            wspieler(SPIELER), write('>'),
            read_integer(VON),  !,
            read_integer(NACH), !,
            write('('), write(VON), write('->'), write(NACH),
            write(')'), nl, !.
zugeingabe(SPIELER,VON,NACH,FARBE,FIGUR) :-
            set(yes),
            wspieler(SPIELER), write('>'),
            read_integer(VON),  !,
            read_integer(NACH), !,
            (VON is 0 ->
               figureingabe(FARBE,FIGUR);
               true
            ),
            write('('), write(VON), write('->'), write(NACH),
            (VON is 0 ->
               (write(' '), wspieler(FARBE), wfigur(FIGUR));
               true
            ),
            write(')'), nl, !.
zugeingabe(_,_,_,_,_) :-
            read_atom(end), halt.
zugeingabe(SPIELER,VON,NACH,FARBE,FIGUR) :-
            write('bitte den Zug als 2 Zahlen eingeben: VON NACH'), nl,
            zugeingabe(SPIELER,VON,NACH,FARBE,FIGUR).

figureingabe(FARBE,FIGUR) :-
            repeat,
            write(' figur(z.B.''wK'')>'),
            read_atom(KURZ),
            kurzlang(KURZ,FARBE,FIGUR).


kurzlang(wK,we,k). kurzlang(wk,we,k).   kurzlang(sK,sw,k). kurzlang(sk,sw,k).
kurzlang(wD,we,d). kurzlang(wd,we,d).   kurzlang(sD,sw,d). kurzlang(sd,sw,d).
kurzlang(wT,we,t). kurzlang(wt,we,t).   kurzlang(sT,sw,t). kurzlang(st,sw,t).
kurzlang(wL,we,l). kurzlang(wl,we,l).   kurzlang(sL,sw,l). kurzlang(sl,sw,l).
kurzlang(wS,we,s). kurzlang(ws,we,s).   kurzlang(sS,sw,s). kurzlang(ss,sw,s).
kurzlang(wB,we,b). kurzlang(wb,we,b).   kurzlang(sB,sw,b). kurzlang(sb,sw,b).

spielername(we,'Weiss').
spielername(sw,'Schwarz').


/* Ist ein eingegebener Zug legal ? */
zuglegal(STELLUNG,SPIELER,0,0) :- !,
            computerzug(STELLUNG,SPIELER,VON,NACH,WERT),
            format('~nVorschlag des Computers: %d->%d (wert=%d)~n',[VON,NACH,WERT]), fail.
zuglegal(_,_,VON,_) :-
            not(feld(_,VON)),  !,
            format('! Das Feld %d gibt es nicht~n',[VON]),  fail.
zuglegal(_,_,_,NACH) :-
            not(feld(_,NACH)), !,
            format('! Das Feld %d gibt es nicht~n',[NACH]), fail.
zuglegal(STELLUNG,SPIELER,VON,_) :-
            not(figur(STELLUNG,SPIELER,_,VON)), !,
            format('! Auf dem Feld %d steht keine Figur von ',[VON]), wspieler(SPIELER), nl, fail.
zuglegal(_,_,VON,VON) :-
            !, write('! Das ist doch kein Zug'), nl, fail.
zuglegal(STELLUNG,SPIELER,_,NACH) :-
            figur(STELLUNG,SPIELER,_,NACH), !,
            format('! Auf dem Feld %d steht schon eine Figur von ', [NACH]), wspieler(SPIELER), nl, fail.
zuglegal(STELLUNG,SPIELER,VON,NACH) :-
            figur(STELLUNG,SPIELER,FIGUR,VON), not(kannziehen(STELLUNG,FIGUR,VON,NACH)), !,
            write('! Das ist kein erlaubter Zug fuer '), wfigur(FIGUR), write(' von '), wspieler(SPIELER), nl, fail.
zuglegal(_,_,_,_).


/* Figuren entfernen oder hinzufuegen:
   setze(STELLUNG,VON,NACH,FARBE,FIGUR,STELLUNGNEU)
   VON = 0 : hinzufuegen von FARBE/FIGUR auf NACH
   NACH= 0 : entfernen von VON                     */
setze(_,VON,0,_,_,_) :-
            not(feld(_,VON)), !,
            format('! Das Feld %d gibt es nicht~n',[VON]), fail.
setze(STELLUNG,VON,0,_,_,_) :-
            not(figur(STELLUNG,_,_,VON)), !,
            format('! Auf dem Feld %d steht keine Figur~n',[VON]), fail.
setze(STELLUNG,VON,0,_,_,STELLUNGNEU) :- 
            figur(STELLUNG,FARBE,FIGUR,VON),
            delete(STELLUNG,f(FARBE,FIGUR,VON),STELLUNGNEU).
setze(_,0,NACH,_,_,_) :-
            not(feld(_,NACH)), !,
            format('! Das Feld %d gibt es nicht~n',[NACH]), fail.
setze(STELLUNG,0,NACH,_,_,_) :-
            figur(STELLUNG,_,_,NACH), !,
            format('! Auf dem Feld %d steht schon eine Figur~n',[NACH]), fail.
setze(STELLUNG,0,NACH,FARBE,FIGUR,[f(FARBE,FIGUR,NACH)|STELLUNG]).


/* Ausgabe der aktuellen (S)tellung: */
wspieler(FARBE) :- sub_atom(FARBE,0,1,_,FA), farbe(FARBE), write(FA).

wfigur(FIGUR) :- lower_upper(FIGUR,FI), write(FI).

ansifarbe(we) :- put_code(27), write('[1;37m').
ansifarbe(sw) :- put_code(27), write('[0;37m').
ansifarbe(ro) :- put_code(27), write('[1;31m').

farbe(F) :- ansi(yes), !, ansifarbe(F).
farbe(_).

feldfarbe(FELD) :- ansi(yes), !, feld(F,FELD), ansifarbe(F).
feldfarbe(_).

schlagausgabe([],_,_) :-
            !.
schlagausgabe([FIGUR],SPIELER,NACH) :- 
            wspieler(SPIELER), wfigur(FIGUR),
            format(' auf %d geschlagen ~2n',[NACH]).


imschachhinweis(STELLUNG,SPIELER) :-
            imschach(STELLUNG,SPIELER), !,
            matthinweis(STELLUNG,SPIELER),
            spielername(SPIELER,NAME),
            write(NAME), write(' steht im Schach !'), nl.
imschachhinweis(_,_).
            

matthinweis(STELLUNG,SPIELER) :-
            matt(STELLUNG,SPIELER), !,
            spielende(0,SPIELER,'Matt'),
matthinweis(_,_).            


spielende(0,_,'Patt') :- !,
            nl, farbe(ro), write('Patt'), farbe(we), nl,
            halt.
spielende(0,SPIELER,TEXT) :- !,
            spielername(SPIELER,NAME),
            nl, farbe(ro), write(NAME),
            write(' ist '), write(TEXT), farbe(we), nl,
            halt.
spielende(_,_,_).


zugausgabe(NR,SPIELER,STELLUNG,VON,NACH,WERT) :-
            figur(STELLUNG,SPIELER,FIGUR,VON),
            nl, nl, wspieler(SPIELER),
            format(' %d. ', [NR]),
            wfigur(FIGUR),
            format(' %d->%d', [VON,NACH] ),
            wertausgabe(SPIELER,WERT).


wertausgabe(SPIELER,_) :- mensch(SPIELER), !, nl, nl.
wertausgabe(_,WERT) :- format('  wert=%d~n~n', [WERT] ).


/* Debug-Ausgaben: */
dbgliste(SPIELER,ZUGLISTE,TIEFE) :- dbg(yes), !,
            wspieler(SPIELER), format(' (%d) :: ', [TIEFE]), write(ZUGLISTE), nl.
dbgliste(_,_,_).


dbgzug(SPIELER,FIGUR,VON,NACH,TIEFE) :- dbg(yes), !,
            wspieler(SPIELER), wfigur(FIGUR), format(' %3d->%3d (%d)~n', [VON,NACH,TIEFE]).
dbgzug(_,_,_,_,_).


dbgbester(SPIELER,z(FIGUR,VON,NACH),WERT,TIEFE) :- dbg(yes), !,
            wspieler(SPIELER), format(' best=%4d ',[WERT]), wfigur(FIGUR), format(' %3d->%3d (%d)~n', [VON,NACH,TIEFE]).
dbgbester(_,_,_,_).


dbgwert(NAME,WERT,TIEFE) :- dbg(yes), !,
            format(' %5s=%4d (%d)~n', [NAME,WERT,TIEFE]).
dbgwert(_,_,_).


dbgalphaschnitt(z(FIGUR,VON,NACH),WERT,ALPHA,TIEFE) :- dbg(yes) , !,
            format(' alpha schnitt : wert=%d < alpha=%d ',[WERT,ALPHA]),
            dbgzug(sw,FIGUR,VON,NACH,TIEFE).
dbgalphaschnitt(_,_,_,_).

dbgbetaschnitt(z(FIGUR,VON,NACH),WERT,BETA,TIEFE) :- dbg(yes) , !,
            format(' beta  schnitt : wert=%d > beta =%d ',[WERT,BETA]),
            dbgzug(we,FIGUR,VON,NACH,TIEFE).
dbgbetaschnitt(_,_,_,_).



/* Initialisierung */
info :- farbe(we),
        write('Hexagon Chess 0.2 -- fr 2003'), nl,
        write(' Zugeingabe numerisch : VON NACH'), nl,
        write(' Beenden mit ''end'''), nl, nl.


usage :- write('AUFRUF: chess3{f|w} [Optionen]'), nl,
         write(' Optionen:  noansi = keine Farben'), nl,
         write('            s      = Schwarz beginnt'), nl,
         write('            x      = Ein- und Aussetzen von Figuren erlaubt'), nl,
         write('            a...   = Anfangsstellung (anormal, a0, atest)'), nl,
         write('            mm     = Mensch   Weiss + Mensch   Schwarz'), nl,
         write('            mc     = Mensch   Weiss + Computer Schwarz (default)'), nl,
         write('            cm     = Computer Weiss + Mensch   Schwarz'), nl,
         write('            cc     = Computer Weiss + Computer Schwarz'), nl,
         write('            debug  = Ausgabe interner Informationen'), nl,
         write('            help   = Ausgabe dieses Textes'), nl,
         halt.


optionen([]).
optionen([noansi|T]) :- !, retractall(ansi(_)),    asserta(ansi(no)),    optionen(T).
optionen([s|T])      :- !, retractall(beginner(_)),asserta(beginner(sw)),optionen(T).
optionen([x|T])      :- !, retractall(set(_)),     asserta(set(yes)),    optionen(T).
optionen([a0|T])     :- !, retractall(anf(_)),     asserta(anf(a0)),     optionen(T).
optionen([anormal|T]):- !, retractall(anf(_)),     asserta(anf(anormal)),optionen(T).
optionen([atest|T])  :- !, retractall(anf(_)),     asserta(anf(atest)),  optionen(T).
optionen([mm|T])     :- !, retractall(mensch(_)),  asserta(mensch(we)),  asserta(mensch(sw)), optionen(T).
optionen([mc|T])     :- !, retractall(mensch(_)),  asserta(mensch(we)),  optionen(T).
optionen([cm|T])     :- !, retractall(mensch(_)),  asserta(mensch(sw)),  optionen(T).
optionen([cc|T])     :- !, retractall(mensch(_)),  optionen(T).
optionen([debug|T])  :- !, retractall(dbg(_)),     asserta(dbg(yes)),    optionen(T).
optionen([help|_])   :- !, info, usage.
optionen([ARG|_])    :- write('unbekannte Option: '), write(ARG), nl, usage.


init(ANFSTELLUNG,BEGINNER) :-
            set_prolog_flag(syntax_error,fail),
            argument_list(ARGS), optionen(ARGS),
            anf(AX), anfang(AX,ANFSTELLUNG),
            beginner(BEGINNER).


/* *** DAS SPIEL *** */
spiel :-
            init(STELLUNG,BEGINNER),
            info,
            ausgabe(STELLUNG),
            schach(0,BEGINNER,STELLUNG).


schach(NR,SPIELER,STELLUNG) :-
            set(no),
            mensch(SPIELER), !,
            repeat,              /* bis Eingabe korrekt */
            zugeingabe(SPIELER,VON,NACH,_,_),
            zuglegal(STELLUNG,SPIELER,VON,NACH), !,
            machezug(NR,SPIELER,STELLUNG,VON,NACH,_).
schach(NR,SPIELER,STELLUNG) :-
            set(yes),
            mensch(SPIELER), !,
            repeat,
            zugeingabe(SPIELER,VON,NACH,FARBE,FIGUR),
            (0 is VON * NACH ->
               setzefigur(NR,SPIELER,STELLUNG,VON,NACH,FARBE,FIGUR);
               (zuglegal(STELLUNG,SPIELER,VON,NACH), !,
                machezug(NR,SPIELER,STELLUNG,VON,NACH,_))
            ).
schach(NR,SPIELER,STELLUNG) :-
            computerzug(STELLUNG,SPIELER,VON,NACH,WERT), !,
            machezug(NR,SPIELER,STELLUNG,VON,NACH,WERT).


machezug(NR,SPIELER,STELLUNG,VON,NACH,WERT) :-
            nummerinc(SPIELER,NR,NR1),
            zugausgabe(NR1,SPIELER,STELLUNG,VON,NACH,WERT),
            zug(STELLUNG,VON,NACH,STELLUNGNEU,SCHLAGLISTE),
            gegner(SPIELER,GEGNER),
            schlagausgabe(SCHLAGLISTE,GEGNER,NACH),
            ausgabe(STELLUNGNEU),
            imschachhinweis(STELLUNGNEU,SPIELER), !,
            schach(NR1,GEGNER,STELLUNGNEU).


setzefigur(NR,SPIELER,STELLUNG,VON,NACH,FARBE,FIGUR) :-
            setze(STELLUNG,VON,NACH,FARBE,FIGUR,STELLUNGNEU),
            ausgabe(STELLUNGNEU), !,
            schach(NR,SPIELER,STELLUNGNEU).


nummerinc(we,NR,NR1) :- NR1 is NR + 1.
nummerinc(sw,NR,NR).


/* *** Ende *** */

