/* putch() with ANSI colors for WinNT - fr 2003
   fputc() with esc sequences filtered out      */

#include <stdio.h>
#include <windows.h>

#define ESC 27
#define LF  10
#define MAXESCLEN 20


static int getdec( int c, char ** pformat )
/* ------------------------------------- */
{
int n;

   n = 0;
   for(;;)
   {
      if( c < '0' || c > '9' )
         break;    /* not a dec digit --> end of dec number */
      n = n * 10 + c - '0';
      c = *(*pformat)++;
   }
   (*pformat)--;
   return n;
}


int putch( int c )
/* ===================================== */
{
static HANDLE hcon = 0;
static char esc[MAXESCLEN];
static int  escix = -1;   /* index in esc[] or -1 for not in esc sequence */

DWORD i, n;
BOOL rc;
WORD att;
char * e;
CONSOLE_SCREEN_BUFFER_INFO scrinfo;

   if( !hcon )
      hcon = GetStdHandle(STD_OUTPUT_HANDLE);

   if( escix < 0 )
   {                                   /* not inside ESC sequence */
      if( c == ESC )
      {
         escix = 0;
         rc = 0;
      }
      else
      {
         if( c == LF )  /*  saving the attribute before scrolling ... */
         {
            scrinfo.wAttributes = FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
            GetConsoleScreenBufferInfo( hcon, &scrinfo );
         }
         rc = WriteConsole( hcon, &c, 1, &n, 0 );
         if( c == LF )
         {              /* ... and restore after scrolling */
            SetConsoleTextAttribute( hcon, scrinfo.wAttributes );
         }              /* otherwise Windows make strange attribute settings */
      }
   }
   else
   {                                   /* inside ESC sequence */
      if( escix >= MAXESCLEN )
         escix = -1;                   /* too long -> back to normal */

      esc[escix++] = c;
      switch( c )
      {
      case 'm':  /* esc[a;a; ...m        set attributes      */
      if( esc[0] == '[' )
      {
         rc = GetConsoleScreenBufferInfo( hcon, &scrinfo );
         if( rc )
            att = scrinfo.wAttributes;
         else
            att = 0;
         esc[escix] = 0; /* terminate */
         e = esc + 1;
         for(;;)
         {
            switch( getdec( '0', &e ) )
            {
            case 0:                 /* attributes off            */
               att = FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
               break;
            case 1:                 /* high intensity            */
               att |= FOREGROUND_INTENSITY;
               break;
            case 2:                 /* normal intensity          */
               att &= ~FOREGROUND_INTENSITY;
               break;

            // case 5:                 /* blink                     */
            // case 7:                 /* reverse video             */
            // case 8:                 /* invisible                 */
               
                                    /* set fore ground color     */
            case 30:
               att &= ~(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
               break;
            case 31:
               att &= ~(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
               att |= FOREGROUND_RED;
               break;
            case 32:
               att &= ~(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
               att |= FOREGROUND_GREEN;
               break;
            case 33:
               att &= ~(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
               att |= FOREGROUND_RED | FOREGROUND_GREEN;
               break;
            case 34:
               att &= ~(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
               att |= FOREGROUND_BLUE;
               break;
            case 35:
               att &= ~(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
               att |= FOREGROUND_RED | FOREGROUND_BLUE;
               break;
            case 36:
               att &= ~(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
               att |= FOREGROUND_GREEN | FOREGROUND_BLUE;
               break;
            case 37:
               att |= FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
               break;

                                    /* set back ground color     */
            case 40:
               att &= ~(BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE);
               break;
            case 41:
               att &= ~(BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE);
               att |= BACKGROUND_RED;
               break;
            case 42:
               att &= ~(BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE);
               att |= BACKGROUND_GREEN;
               break;
            case 43:
               att &= ~(BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE);
               att |= BACKGROUND_RED | BACKGROUND_GREEN;
               break;
            case 44:
               att &= ~(BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE);
               att |= BACKGROUND_BLUE;
               break;
            case 45:
               att &= ~(BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE);
               att |= BACKGROUND_RED | BACKGROUND_BLUE;
               break;
            case 46:
               att &= ~(BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE);
               att |= BACKGROUND_GREEN | BACKGROUND_BLUE;
               break;
            case 47:
               att |= BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE;
               break;
            }
            if( *e == 'm' )
               break;
            if( *e != ';' )
               goto wrongesc;
            e++;
         }
         SetConsoleTextAttribute( hcon, att );
         escix = -1;
      }
      else
         goto wrongesc;
      break;

      /* erlaubte Zeichen in escape-Sequenzen: */
      case '[': case ';':
      case '0': case '1': case '2': case '3': case '4':
      case '5': case '6': case '7': case '8': case '9':
         break;
            
      /* UNerlaubte Zeichen in escape-Sequenzen: */
      case '\033' : /* esc in esc sequence not allowed */
         escix--;   /* damit nicht esc ausgegeben wird ->
                       wuerde recursiv wieder in esc-Modus gehen */
            /* fall through */
      default:      /* ungueltige Sequenz -> ausgeben: */
      wrongesc:
         n = escix;
         escix = -1;
         for( i = 0; i < n; i++ )
         {
            WriteConsole( hcon, esc+i, 1, &n, 0 );
         }
      }
      rc = 1;
   }
   return rc ? c : EOF;
}


int fputc( int c, FILE * stream )
/* ===================================== */
{
static char inesc = 0;

int l;
   
   if( !inesc )
   {                                   /* not inside ESC sequence */
      if( c == ESC )
      {
         inesc = 1;
      }
      else
      {
         l = fwrite( &c, 1, 1, stream );
         if( l != 1 )
            return EOF;
      }
   }
   else
   {                                   /* inside ESC sequence */
      switch( c )
      {
      case 'm':  /* esc[a;a; ...m        set attributes      */
         inesc = 0;
         break;
         
      /* erlaubte Zeichen in escape-Sequenzen: */
      case '[': case ';':
      case '0': case '1': case '2': case '3': case '4':
      case '5': case '6': case '7': case '8': case '9':
         break;
            
      /* UNerlaubte Zeichen in escape-Sequenzen: */
      default:
         inesc = 0;
      }
   }
   return c;
}


